$title	Generate a Fan Diagram

$if not set item $set item output 

*.$set plotscn yes

$if not set t    $set t t
$if not set s    $set s s
$if not set tl   $set tl tl
$if not set data $set data fan
$if not set item $set item v
$if not set prob $set prob prob

set	n	Set of possible percentiles /0*100/,
	k	Ordering set for scenarios  /1*100/;

set	s(*)	States of world in the time path,
	t(*)	Time periods;

$gdxin '%data%.gdx'
$load t=%t% s=%s% 

set	pct(n)	Percentiles to be evaluated /10,20,30,40,50,60,70,80,90/,
	tl(t)	Labels for the time periods;

alias (s,sw,ssw), (s,sc1,sc2);

set		active(sw,t)	Active scenarios by time period,
		rmap(k,sw)	Mapping from index to scenario,
		sm(t,sw,ssw)	Associates state sw with active state ssw in time period t;

parameter	vt(t,sw)
		prob(sw)	Probability weights;

$load vt=%item% tl=%tl% prob=%prob%

*	This version of the program is based on non-stochastic input:

sm(t,s,s) = yes;

parameter	v(sw)		Values in current period, 
		r(sw)		Scenario rank in current period,
		pi(sw)		Probability weights in current period,
		p(n)		Percentile values to be computed,
		pctl(sw)	Percentile estimates

set		sc(sw);

abort$round(sum(sw,prob(sw))-1,5)	"Scenarios probability weights (prob) do not sum to unity";
abort$(card(sw) > card(k))	"Too many states of world -- need to increase dimensions in set k";

*	Blank invocation to initialize parameters

$batinclude rank


*	Initialize by making sure that scenarios sum to unity:

prob(sw) = prob(sw)/sum(ssw,prob(ssw));
p(n)$pct(n) = ord(n)-1;

parameter	pctlmin		Minimum value in current period
		pctlmax		Maximum value in current period
		pvalue		Cumulative probability value
		values		Values in current period,
		vpct		Percentile plots,
		alpha		Alpha values
		minmaxlog	Iteration log
		ranklog		Ranking log
		itlog		Iteration log
		epsilon		Perturbation parameter /1e-4/;

loop(t,
	sc(sw) = yes$sum(sm(t,ssw,sw),1);
	v(sc) = vt(t,sc);

$batinclude rank v sc r

	ranklog(t,sc) = r(sc);
	rmap(k,sc) = no;
	rmap(k,sc)$(ord(k)<=card(sc)) = yes$(ord(k)=r(sc));
	pi(sc) = sum(sm(t,sw,sc), prob(sw));
*.	display sc,pi,rmap;
	pvalue = 0;  pctl(sc) = 0;
	loop(k$(ord(k)<=card(sc)),
	  loop(rmap(k,sc), 
	    pctl(sc) = 100 * (pvalue + pi(sc)/2);
*.	    display pctl;
	    pvalue = pvalue + pi(sc);
	  );
	);
	pctlmin = smin(sc, pctl(sc));
	pctlmax = smax(sc, pctl(sc));

	minmaxlog(t,sc) = pctl(sc) + eps;
	minmaxlog(t,"min") = pctlmin;
	minmaxlog(t,"max") = pctlmax;

	values(t,k,sc,"v")$rmap(k,sc) = v(sc);
	values(t,k,sc,"pi")$rmap(k,sc) = 100*pi(sc);
	values(t,k,sc,"pctl")$rmap(k,sc) = pctl(sc);

	loop(pct,
	  if (p(pct) <= pctlmin+epsilon,
	    loop(rmap(k,sc)$(ord(k)=1),  
	      vpct(t,pct) = v(sc);
	      itlog(t,pct,sc,sc,"p") = p(pct);
	      itlog(t,pct,sc,sc,"pctl1") = pctl(sc);
	      itlog(t,pct,sc,sc,"pctl2") = pctl(sc);
	      itlog(t,pct,sc,sc,"alpha") = 0;);
	  elseif p(pct) >= pctlmax-epsilon,
	    loop(rmap(k,sc)$(ord(k) eq card(sc)),  
	      vpct(t,pct) = v(sc);
	      itlog(t,pct,sc,sc,"p") = p(pct);
	      itlog(t,pct,sc,sc,"pctl1") = pctl(sc);
	      itlog(t,pct,sc,sc,"pctl2") = pctl(sc);
	      itlog(t,pct,sc,sc,"alpha") = 1;);
	  else 
	    loop(rmap(k,sc1), loop(sc2$(rmap(k+1,sc2) and
	                     (p(pct) >= pctl(sc1)-epsilon) and 
			     (p(pct) <= pctl(sc2)+epsilon)), 
	      alpha = (p(pct)-pctl(sc1))/(pctl(sc2)-pctl(sc1));
	      itlog(t,pct,sc1,sc2,"p") = p(pct);
	      itlog(t,pct,sc1,sc2,"pctl1") = pctl(sc1);
	      itlog(t,pct,sc1,sc2,"pctl2") = pctl(sc2);
	      itlog(t,pct,sc1,sc2,"alpha") = alpha;
	      vpct(t,pct) = alpha * v(sc2) + (1-alpha) * v(sc1);
	    ));
	  );
	);
);
execute 'if exist rank_data.gdx rm rank_data.gdx';
execute 'if exist rank_output.gdx rm rank_output.gdx';

display minmaxlog, ranklog;
option values:1:2:1;
display values;
option itlog:1:3:1;
display itlog;

parameter	xlpct(t,pct)	Exel representation of the data;
xlpct(t,pct) = vpct(t,pct);
xlpct(t,pct+1) = xlpct(t,pct+1) - xlpct(t,pct);
*.execute_unload 'xlfan.gdx',xlpct,t,tl,pct,vt,sc;
*.execute 'gdxxrw i=xlfan.gdx o=xlfan.xls par=xlpct';

*	Plotting begins here:

file	kplot /probplot.gp/,
	kprb  /probplot.dat/,
	kscn  /scnplot.dat/;
put kprb;
kprb.nr=2; kprb.nw=13; kprb.nd=6;
loop(t,
	put (ord(t)-1); loop(pct, put vpct(t,pct);); put /;
);
putclose;

put kscn;
kscn.nr=2; file.nw=12; file.nd=8;
loop(t,
	loop(sw, put vt(t,sw);); put /;
);
putclose;

put kplot; kplot.lw=0; kplot.nr=0; kplot.nw=0; kplot.nd=0;
put 'set xrange[1:',(card(t)-1):0,']'/;
put 'unset key'/;
put 'set tics out'/;
put 'set border 3'/;
put 'set grid front'/;
put "set title '",vt.ts,"'"/;
put 'set xtics nomirror'/;
put 'set ytics nomirror'/;
put 'set xzeroaxis'/;
put 'set yzeroaxis'/;
put 'set xtics (';
file.nd = 0; file.nw = 0; scalar ntic/0/;
loop(t, if (tl(t),
    if (ntic gt 0, put ',';);
    put '\'/' "',tl.te(t),'" ',ord(t);
    ntic = ntic + 1
)); 
put ')'/;
put 'set palette gray'/;
put 'set style fill pattern 2'/;
put 'set style line 1 lt palette frac 0.95'/;
put 'set style line 2 lt palette frac 0.85'/;
put 'set style line 3 lt palette frac 0.75'/;
put 'set style line 4 lt palette frac 0.50'/;
put 'set style line 5 lt 0 lw 1 lc rgb "black"'/;
put 'unset colorbox'/;
put	'plot \'/
	' "probplot.dat" u 1:2:3 w filledcu below ls 1 title "0.1" , \'/
	'"" u 1:3:4 w filledcu below ls 2 title "0.2" , \'/
	'"" u 1:4:5 w filledcu below ls 3 title "0.3" , \'/
	'"" u 1:5:6 w filledcu below ls 4 title "0.4" , \'/
	'"" u 1:6:7 w filledcu below ls 4 title "0.5" , \'/
	'"" u 1:7:8 w filledcu below ls 3 title "0.6" , \'/
	'"" u 1:8:9 w filledcu below ls 2 title "0.7" , \'/
	'"" u 1:9:10 w filledcu below ls 1 title "0.8", \'/;
$if set plotscn loop(sw, put '"scnplot.dat"  using ',ord(sw):0:0,' with lines ls 5, \'/;)
	put '"probplot.dat" using 1:6 with lines ls 5'/;
putclose;

execute 'wgnuplot probplot.gp -';

