$title	Construct Stochastic Logic Arrays Based on State Transition Points

set	t(*)	Time periods
	sw(*)	States of world;

$gdxin 'probtree.gdx'	

$load t sw 

set	transition(t,sw,sw)	User-supplied state transitions;

$load transition 

alias (sw, ssw, sow), (tp,t,ttp);

set	eq(tp,sw)	Indicates that state sw is active in time period tp,
	st(t,sw,sw)	State transitions 
	sm(tp,sw,sow)	Associates state sw with active state sow in time period tp,
	tfirst(t)	First period
	tlast(t)	Last period

	as(sw)		Array keeping track of active states,
	bs(sw)		Array keeping track of branching states,
	ns(sw)		Array keeping track of new states,

	error(tp,sw)	Indices of states in which error is detected,

	aslog(tp,sw)	Log report of active states,
	bslog(tp,sw)	Log report of branching states,
	nslog(tp,sw)	Log report of new states;

parameter
	column(sw)	Column index for displaying the state,
	cindex		Counter for column index;  

*	Identify the first and last time periods:

tfirst(t) = yes$(ord(t) eq 1);
tlast(t) = yes$(ord(t) eq card(t));


*	Begin with the user-specified transition points:

st(t,sw,sow) = transition(t,sw,sow);

*	No states are initially tagged as active:

eq(tp,sw) = no;

*	Look at the state transitions first to verify that the event tree
*	is valid:


loop(tp,

*	Identify the states of world from which the event tree 
*	branches in this period:

	bs(sw) = yes$sum(st(tp,sw,sow),1);
	ns(sw) = yes$sum(st(tp,sow,sw),1);

*	Define the set of states which are currently active:

	as(sw) = eq(tp,sw);
	bslog(tp,bs) = yes;
	nslog(tp,ns) = yes;
	aslog(tp,as) = yes;

*	We have not reached the first branch in the event tree:

	if (card(as)=0,
	  if (card(ns) > 0,
	    error(tp,sw) = yes$sum(st(tp,sw,sow),1);
	    abort$(card(bs)>1) "Root node of event tree must be a single state:", error, bslog, nslog, aslog;
	    error(tp,sw) = no;
	    as(bs) = yes;
	    eq(ttp,bs) = yes;
	    column(bs) = 1;
	    sm(ttp,ns,bs) = yes$(ord(ttp)<=ord(tp));
	    sm(ttp,bs,bs) = yes;
	  );
	else
	  error(tp,bs) = yes$(not as(bs));
	  abort$card(error) "Branch in event tree originates from an inactive state:", error, bslog, nslog, aslog;

	  error(tp,ns) = yes$(sum(st(tp,bs,ns),1) <> 1);
	  abort$card(error) "State is associated with more than one branch:", error, bslog, nslog, aslog;

	  st(tp,bs,bs) = yes;
	);
	as(ns) = yes;
	eq(tp+1,as) = yes;
	sm(ttp,ns,ns)$(ord(ttp) > ord(tp)) = yes;
	loop(bs,
	  cindex = column(bs) + 1;
	  loop(st(tp,bs,ns),
	    column(sw)$(column(sw) ge cindex)  = column(sw) + 1;
	    column(ns) = cindex;
	    cindex = cindex + 1;
	  ));
);
display eq;

loop(tp, loop(st(tp,sw,sow),
	  sm(ttp,sow,ssw)$(sm(ttp,sw,ssw) and (not eq(ttp,sow))) = yes;));
display column;

st(tp,sw,sow) = yes$(sm(tp,sow,sw) and eq(tp+1,sow));
option eq:0:1:1;
display eq;
option sm:0:0:1;
display sm;
option st:0:0:1;
display st;

