$title	Plotting Routine which Displays Stochastic Structure

$if not set piscale $set piscale  no

set	sw(*)	States of world,
	t(*)	Time periods to be plotted

$gdxin 'treeplot.gdx'
$load t sw 

alias (sw, sow), (t,tp);

set	eq(tp,sw)	Equilibrium structure (sw is active in time period tp),
	sm(tp,sw,sow)	State matching (sw is represented by state sow in time period tp)
	tl(t)		Labels for the plot;
	
parameter	pi(sw)	State probability;

$load sm eq tl

$if %piscale%==yes $load pi
$if %piscale%==no  pi(sw) = 1/card(sw);

set		tlast(t)	Last period;

alias (t,tp);

tlast(t) = yes$(ord(t)=card(t));

parameter	tree(tp,sw)	Stochastic structure;
set sc(sw);
scalar pct;
tree(t,sw) = na;
loop(t,
	sc(sw) = yes$sum(sm(t,sow,sw),1);
	pct = 1;
	loop(sc,
	  if (%piscale%,
	    tree(t,sc) = 100 * (pct - sum(sm(t,sw,sc),pi(sw)) / 2);
	    pct = pct - sum(sm(t,sw,sc),pi(sw));
	  else
	    tree(t,sc) = 100 * (pct - sum(sm(t,sw,sc),1)/card(sw) / 2);
	    pct = pct - sum(sm(t,sw,sc),1)/card(sw);
	  );
	);
	tree(t,sw)$(eq(t,sw) or eq(t+1,sw)) = sum(sm(t,sw,sc), tree(t,sc));
);

file	kplt /probtree.gp/,
	kdat /probtree.dat/;

scalar gp_na/na/;
kdat.nw = 16;
kdat.nd = 8;
kdat.nr = 2;
put kdat;
loop(sw,
    loop(t,
      put ord(t);
      if (tree(t,sw) eq gp_na,
        put '          ';
      else
        put tree(t,sw);
      );
      put /;
    );
    put //;
);
putclose;

kplt.lw=0;
kplt.nr=1;kplt.nd=0;
put kplt,
	'set title "Stochastic structure"'/
	'set xrange [1:',(card(t)+1):0,']'/
	'set border 5 '/
	'set xtics nomirror'/
	'unset yzeroaxis'/
	'unset xzeroaxis'/
	'unset ytics'/
	'unset key'/
	'set data style lines'/
	'set yrange [-10:110]'/;

put 'set xtics (';
file.nd = 0; file.nw = 0; scalar ntic/0/;
loop(t, if (tl(t),
    if (ntic gt 0, put ',';);
    put '\'/' "',tl.te(t),'" ',ord(t);
    ntic = ntic + 1
)); 
put ')'/;

parameter	lblloc(sw)	Label locations;
lblloc(sw) = sum(tlast(t), tree(t,sw));

file.nr=1;file.nd=1;
loop(sw, put 'set label "',sw.tl,'" at ',(card(t)+0.5),',',lblloc(sw)/;); 

put	'set term postscript landscape color "Times-Roman" 12'/
	"set output 'probtree.eps'"/;

file.nd=0;
put	'plot \'/;
loop(sw,
	put '"probtree.dat" index ',(ord(sw)-1):0,' using 1:2 "%lf%lf"';
	if (ord(sw) ne card(sw), put ',\'/; else put /; );
);

*	Plot on the screen after having generated the EPS file:
put	'set term windows'/
	'set output'/
	'replot'/;
putclose;

execute 'wgnuplot probtree.gp -';

